# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2013 credativ (http://www.credativ.co.uk). All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv, fields
import time
from redmine_connector.redmine_osv import Connection

import logging
_logger = logging.getLogger(__name__)
_logger.setLevel(logging.DEBUG)

DEBUG = True

class project_match_wizard(osv.osv_memory):
    _name = 'project.match.wizard'

    _columns = {
        'referential_id': fields.many2one(
            'external.referential',
            string='Redmine referential'),
        'projects': fields.one2many(
            'project.match.line', 'match_wizard_id',
            string='Projects'),
    }

    def default_get(self, cr, uid, fields, context=None):
        if 'active_id' in context and 'active_model' in context and context['active_model'] == 'external.referential':
            referential_id = context['active_id']
        else:
            _logger.error('Redmine: project_match_wizard: no active_model in context: %s' % (str(context),))
            return {}

        referential_pool = self.pool.get('external.referential')
        model_data_pool = self.pool.get('ir.model.data')
        referential = referential_pool.browse(cr, uid, referential_id, context=context)
        OE_MODELS = ['project.project','account.analytic.account']

        res = {'referential_id': referential_id,
               'projects': []}

        # retrieve all the RM projects
        try:
            conn = Connection(referential.location, referential.apiusername, referential.apipass,
                              referential.ar_api_url, referential.ar_api_key)
            projects = conn.call('Projects.find')
        except Exception, e:
            raise osv.except_osv('Connection Error',
                                 'Redmine: could not retrieve redmine projects: %s' % (str(e),))

        # map all the already linked RM projects' IDs to their OE IDs
        rm_project_ids = [project.get_id() for project in projects]
        oe_data_ids =\
            dict([(oe_model,
                   model_data_pool.search(cr, uid,
                                          [('external_referential_id','=',referential.id),
                                           ('name','in',['%s/%s' % (oe_model.replace('.','_'), i)
                                                         for i in rm_project_ids]),
                                           ('model','=',oe_model)],
                                          context=context))
                  for oe_model in OE_MODELS])

        oe_res_ids = {}
        for model, data_ids in oe_data_ids.items():
            oe_res_ids[model] = {}
            for r in model_data_pool.read(cr, uid, data_ids, ['name', 'res_id'], context=context):
                try:
                    ext_id = int(r['name'][r['name'].index('/')+1:])
                except ValueError:
                    _logger.error('Redmine: could not parse external identifier: %s' % (r['name'],))
                    continue
                oe_res_ids[model][ext_id] = r['res_id']

        # construct a one2many list of all the RM projects, including
        # existing OE projects/accounts where available
        oe_pools = dict([(model, self.pool.get(model)) for model in OE_MODELS])
        for rm_project in projects:
            rm_project_id = int(rm_project.get_id())
            oe_resources = dict([(model, oe_pools[model].browse(cr, uid, oe_res_ids[model].get(rm_project_id, None), context=context))
                                 for model in OE_MODELS])

            project_id = oe_resources.get('project.project') and oe_resources['project.project'].id or False
            analytic_account_id = oe_resources.get('project.project') and oe_resources['project.project'].analytic_account_id.id or\
                                  oe_resources.get('account.analytic.account') and oe_resources['account.analytic.account'].id or\
                                  False

            res['projects'].append([0, 0, {'external_referential_id': referential.id,
                                           'rm_project_name': rm_project.attributes['name'],
                                           'rm_project_id': rm_project_id,
                                           'project_id': project_id,
                                           'analytic_account_id': analytic_account_id}])

        return res

    def create(self, cr, uid, vals, context=None):
        if 'projects' in vals:
            # add the match_wizard_id to the default_get projects
            projects = vals.pop('projects')
            res_id = super(project_match_wizard, self).create(cr, uid, vals, context=context)
            self.write(cr, uid, res_id,
                       {'projects': [(a, b, dict([('match_wizard_id', res_id)] + fields.items())) for (a, b, fields) in projects]},
                       context=context)
            return res_id
        else:
            return super(project_match_wizard, self).create(cr, uid, vals, context=context)

    def update_matchings(self, cr, uid, ids, context=None):
        if not context:
            context = {}

        model_data_pool = self.pool.get('ir.model.data')
        referential_pool = self.pool.get('external.referential')
        OE_MODELS = ['project.project','account.analytic.account']

        for matches in self.browse(cr, uid, ids, context=context):
            for match in matches.projects:
                # Simulate base_external_referentials import; ensure
                # OE project/analytic account are correctly associated
                # with RM project

                # FIXME In fact, we should do both project.project and account.analytic.account
                models = []
                if match.project_id:
                    models.append({'oe_model': 'project.project', 'oe_id': match.project_id.id})
                if match.analytic_account_id:
                    models.append({'oe_model': 'account.analytic.account', 'oe_id': match.analytic_account_id.id})
                if not match.project_id and not match.analytic_account_id:
                    # in this case, the matching may have been
                    # explicitly removed
                    models.append({'oe_model': None, 'oe_id': None})

                referential = referential_pool.browse(cr, uid, match.external_referential_id.id, context=context)

                for oe_resource in models:
                    oe_model = oe_resource['oe_model']
                    oe_id = oe_resource['oe_id']

                    if oe_model and oe_id:
                        # find any existing OE/RM project/account
                        # association
                        oe_data_ids =\
                            model_data_pool.search(cr, uid, [('external_referential_id','=',referential.id),
                                                             ('name','=','%s/%s' % (oe_model.replace('.','_'), match.rm_project_id)),
                                                             ('model','=',oe_model)],
                                                   context=context)
                    else:
                        # find any existing OE/RM project and account
                        # associations to be removed
                        oe_data_ids =\
                            model_data_pool.search(cr, uid, [('external_referential_id','=',referential.id),
                                                             ('name','in',['%s/%s' % (m.replace('.','_'), match.rm_project_id) for m in OE_MODELS])],
                                                   context=context)

                    if not oe_data_ids and oe_model and oe_id:
                        # if the project/account has not already been
                        # marked as imported, then create an ir.model.data
                        # record for it
                        model_data_pool.create(cr, uid, {'name': '%s/%s' % (oe_model.replace('.','_'), match.rm_project_id),
                                                         'module': 'extref/%s' % (referential.name,),
                                                         'model': oe_model,
                                                         'res_id': oe_id,
                                                         'external_referential_id': referential.id},
                                               context=context)
                                                         
                        # update the redmine_id field on the associated
                        # project/account
                        self.pool.get(oe_model).write(cr, uid, oe_id, {'redmine_id': match.rm_project_id}, context=context)

                    elif len(oe_data_ids) == 1 and oe_model and oe_id:
                        # if the project/account has already been marked
                        # as imported, then update the project name and
                        # the ir.model.data model, res_id, and update
                        # timestamp in case the submitted OE project has
                        # changed
                        model_data_pool.write(cr, uid, oe_data_ids[0],
                                              {'model': oe_model,
                                               'res_id': oe_id,
                                               'date_update': time.strftime('%Y-%m-%d %H:%M:%S')},
                                              context=context)

                        # update the redmine_id field on the associated
                        # project/account
                        self.pool.get(oe_model).write(cr, uid, oe_id, {'redmine_id': match.rm_project_id}, context=context)

                    elif oe_data_ids and not oe_model and not oe_id:
                        # in this case, the existing RM/OE project
                        # associations should be removed

                        # unset redmine_id values for project/account
                        for res in model_data_pool.read(cr, uid, oe_data_ids, ['model', 'res_id'], context=context):
                            self.pool.get(res['model']).write(cr, uid, res['res_id'], {'redmine_id': False}, context=context)

                        # remove the ir.model.data link
                        model_data_pool.unlink(cr, uid, oe_data_ids, context=context)

                    elif oe_model:
                        _logger.error('Redmine: Found multiple ir.model.data records for %s/%s in referential #%s' %\
                                      (oe_model.replace('.','_'), match.rm_project_id, referential.id))

                    else:
                        _logger.warn('Redmine: No project/account specified for RM project #%s ("%s") in referential #%s' %\
                                     (match.rm_project_id, match.rm_project_name, referential.id))

        return {'type': 'ir.actions.act_window_close'}

class project_match_line(osv.osv_memory):
    _name = 'project.match.line'

    _columns = {
        'external_referential_id': fields.many2one(
            'external.referential',
            string='Redmine referential',
            required=True,
            readonly=True),
        'match_wizard_id': fields.many2one(
            'project.match.wizard',
            string='Project match wizard',
            required=True,
            readonly=True),
        'rm_project_name': fields.char(
            size=255,
            string='RM project',
            readonly=True),
        'rm_project_id': fields.integer(
            string='RM project ID',
            readonly=True),
        'project_id': fields.many2one(
            'project.project',
            string='Project'),
        'analytic_account_id': fields.many2one(
            'account.analytic.account',
            string='Analytic accout'),
    }

    def on_select_oe_project(self, cr, uid, ids, project_id, context=None):
        # ensure that non-matching project/analytic account
        # combinations cannot be specified

        if not project_id:
            return {'value': {'project_id': False}}

        project = self.pool.get('project.project').browse(cr, uid, project_id, context=context)
        if project:
            return {'value': {'analytic_account_id': project.analytic_account_id.id}}
        else:
            return {'value': {'analytic_account_id': False}}

    def on_select_oe_analytic_account(self, cr, uid, ids, analytic_account_id, context=None):
        # ensure that non-matching project/analytic account
        # combinations cannot be specified

        if not analytic_account_id:
            return {'value': {'analytic_account_id': False}}

        project_ids = self.pool.get('project.project').search(cr, uid, [('analytic_account_id','=',analytic_account_id)], context=context)
        if len(project_ids) == 1:
            return {'value': {'project_id': project_ids[0]}}
        elif len(project_ids) > 1:
            return {'domain': {'project_id': [('id','in',project_ids)]}}
        else:
            return {'value': {'project_id': False}}
